// -*- C++ -*-
/*!
 * @file  Converter.cpp
 * @brief Converter component
 * $Date$
 *
 * $Id$
 */

#include "Converter.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define BOOL 5
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40
#define BOOLARRAY 50

// Module specification
// <rtc-template block="module_spec">
static const char* converter_spec[] =
  {
    "implementation_id", "Converter",
    "type_name",         "Converter",
    "description",       "Converter component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.InPort_DataType", "double",
    "conf.default.OutPort_DataType", "long",
    "conf.default.Extra_num", "1",

    ""
  };
// </rtc-template>

Converter::Converter(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortinIn("shortin", m_shortin),
    m_longinIn("longin", m_longin),
    m_floatinIn("floatin", m_floatin),
    m_doubleinIn("doublein", m_doublein),
    m_boolinIn("boolin", m_boolin),

    m_sarrinIn("shortarrayin", m_sarrin),
    m_larrinIn("longarrayin", m_larrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),
    m_barrinIn("booleanarrayin", m_barrin),

    m_shortoutOut("shortout", m_shortout),
    m_longoutOut("longout", m_longout),
    m_floatoutOut("floatout", m_floatout),
    m_doubleoutOut("doubleout", m_doubleout),
    m_booloutOut("boolout", m_boolout),

    m_sarroutOut("shortarrayout", m_sarrout),
    m_larroutOut("longarrayout", m_larrout),
    m_farroutOut("floatarrayout", m_farrout),
    m_darroutOut("doublearrayout", m_darrout),
    m_barroutOut("booleanarrayout", m_barrout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("converter", "converter", m_converter);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Converter::~Converter()
{
}


RTC::ReturnCode_t Converter::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("InPort_DataType", m_InPort_DataType, "double");
  bindParameter("OutPort_DataType", m_OutPort_DataType, "long");
  bindParameter("Extra_num", m_Extra_num, "1");
  
  // </rtc-template>
  std::cout << "Converter start." << std::endl;
  return RTC::RTC_OK;
}




RTC::ReturnCode_t Converter::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}



bool svc_flag=false;
int svc_intype=0;
int svc_outtype=0;
int svc_extra=0;
bool activate_flag=true;

RTC::ReturnCode_t Converter::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    intype = svc_intype;
  else{
    if(m_InPort_DataType!="short" && m_InPort_DataType!="long" && m_InPort_DataType!="float" && m_InPort_DataType!="double" && m_InPort_DataType!="bool" && m_InPort_DataType!="shortArray" && m_InPort_DataType!="longArray" && m_InPort_DataType!="floatArray" && m_InPort_DataType!="doubleArray" && m_InPort_DataType!="boolArray"){
      std::cout << "InPort_DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_InPort_DataType=="short")
      intype = SHORT;
    else if(m_InPort_DataType=="long")
      intype = LONG;
    else if(m_InPort_DataType=="float")
      intype = FLOAT;
    else if(m_InPort_DataType=="double")
      intype = DOUBLE;
    else if(m_InPort_DataType=="bool")
      intype = BOOL;
    else if(m_InPort_DataType=="shortArray")
      intype = SHORTARRAY;
    else if(m_InPort_DataType=="longArray")
      intype = LONGARRAY;
    else if(m_InPort_DataType=="floatArray")
      intype = FLOATARRAY;
    else if(m_InPort_DataType=="doubleArray")
      intype = DOUBLEARRAY;
    else if(m_InPort_DataType=="boolArray")
      intype = BOOLARRAY;
  }


  if(svc_flag)
    outtype = svc_outtype;
  else{
    if(m_OutPort_DataType!="short" && m_OutPort_DataType!="long" && m_OutPort_DataType!="float" && m_OutPort_DataType!="double" && m_OutPort_DataType!="bool" && m_OutPort_DataType!="shortArray" && m_OutPort_DataType!="longArray" && m_OutPort_DataType!="floatArray" && m_OutPort_DataType!="doubleArray" && m_OutPort_DataType!="boolArray"){
      std::cout << "OutPort_DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_OutPort_DataType=="short")
      outtype = SHORT;
    else if(m_OutPort_DataType=="long")
      outtype = LONG;
    else if(m_OutPort_DataType=="float")
      outtype = FLOAT;
    else if(m_OutPort_DataType=="double")
      outtype = DOUBLE;
    else if(m_OutPort_DataType=="bool")
      outtype = BOOL;
    else if(m_OutPort_DataType=="shortArray")
      outtype = SHORTARRAY;
    else if(m_OutPort_DataType=="longArray")
      outtype = LONGARRAY;
    else if(m_OutPort_DataType=="floatArray")
      outtype = FLOATARRAY;
    else if(m_OutPort_DataType=="doubleArray")
      outtype = DOUBLEARRAY;
    else if(m_OutPort_DataType=="boolArray")
      outtype = BOOLARRAY;
  }


  if(svc_flag)
    extra = svc_extra;
  else
    extra = m_Extra_num;

  if(extra < 1 || 10 < extra){
    std::cout << "Extra_Num ERROR" << std::endl;
    return RTC::RTC_ERROR;
  }


  switch(intype){
    case SHORT:
      registerInPort("shortin", m_shortinIn);
    break;
    case LONG:
      registerInPort("longin", m_longinIn);
    break;
    case FLOAT:
      registerInPort("floatin", m_floatinIn);
    break;
    case DOUBLE:
      registerInPort("doublein", m_doubleinIn);
    break;
    case BOOL:
      registerInPort("booleanin", m_boolinIn);
    break;
    case SHORTARRAY:
      registerInPort("shortArrayin", m_sarrinIn);
    break;
    case LONGARRAY:
      registerInPort("longArrayin", m_larrinIn);
    break;
    case FLOATARRAY:
      registerInPort("floatArrayin", m_farrinIn);
    break;
    case DOUBLEARRAY:
      registerInPort("doubleArrayin", m_darrinIn);
    break;
    case BOOLARRAY:
      registerInPort("booleanArrayin", m_barrinIn);
    break;
  }

  switch(outtype){
    case SHORT:
      registerOutPort("shortout", m_shortoutOut);
    break;
    case LONG:
        registerOutPort("longout", m_longoutOut);
    break;
    case FLOAT:
        registerOutPort("floatout", m_floatoutOut);
    break;
    case DOUBLE:
        registerOutPort("doubleout", m_doubleoutOut);
    break;
    case BOOL:
        registerOutPort("booleanout", m_booloutOut);
    break;
    case SHORTARRAY:
        registerOutPort("shortArrayout", m_sarroutOut);
    break;
    case LONGARRAY:
        registerOutPort("longArrayout", m_larroutOut);
    break;
    case FLOATARRAY:
        registerOutPort("floatArrayout", m_farroutOut);
    break;
    case DOUBLEARRAY:
        registerOutPort("doubleArrayout", m_darroutOut);
    break;
    case BOOLARRAY:
        registerOutPort("booleanArrayout", m_barroutOut);
    break;
  }

  activate_flag=false;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Converter::onDeactivated(RTC::UniqueId ec_id)
{
  m_shortinIn.read();
  m_longinIn.read();
  m_floatinIn.read();
  m_doubleinIn.read();
  m_boolinIn.read();
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();
  m_barrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Converter::onExecute(RTC::UniqueId ec_id)
{
  bool flag=false;
  short shortnum = 0;
  long longnum = 0;
  float floatnum = 0;
  double doublenum = 0;
  bool boolnum = 0;
  int selement = 0, lelement = 0, felement = 0, delement = 0, belement = 0;
  std::vector<short> sarrnum;
  std::vector<long> larrnum;
  std::vector<float> farrnum;
  std::vector<double> darrnum;
  std::vector<bool> barrnum;

  switch(intype){
    case SHORT:
      if(m_shortinIn.isNew()){
        m_shortinIn.read();
        shortnum = m_shortin.data;
        flag = true;
      }
    break;

    case LONG:
      if(m_longinIn.isNew()){
        m_longinIn.read();
        longnum = m_longin.data;
        flag = true;
      }
    break;

    case FLOAT:
      if(m_floatinIn.isNew()){
        m_floatinIn.read();
        floatnum = m_floatin.data;
        flag = true;
      }
    break;

    case DOUBLE:
      if(m_doubleinIn.isNew()){
        m_doubleinIn.read();
        doublenum = m_doublein.data;
        flag = true;
      }
    break;

    case BOOL:
      if(m_boolinIn.isNew()){
        m_boolinIn.read();
        boolnum = m_boolin.data;
        flag = true;
      }
    break;

    case SHORTARRAY:
      if(m_sarrinIn.isNew()){
        m_sarrinIn.read();
        selement = m_sarrin.data.length();
        if(selement < extra){
          std::cout << "DataLength ERROR." << std::endl;
          return RTC::RTC_ERROR;
        }
        for(int i=0; i<selement; i++)
          sarrnum.push_back(m_sarrin.data[i]);
        flag = true;
      }
    break;

    case LONGARRAY:
      if(m_larrinIn.isNew()){
        m_larrinIn.read();
        lelement = m_larrin.data.length();
        if(lelement < extra){
          std::cout << "DataLength ERROR." << std::endl;
          return RTC::RTC_ERROR;
        }
        for(int i=0; i<lelement; i++)
          larrnum.push_back(m_larrin.data[i]);
        flag = true;
      }
    break;

    case FLOATARRAY:
      if(m_farrinIn.isNew()){
        m_farrinIn.read();
        felement = m_farrin.data.length();
        if(felement < extra){
          std::cout << "DataLength ERROR." << std::endl;
          return RTC::RTC_ERROR;
        }
        for(int i=0; i<felement; i++)
          farrnum.push_back(m_farrin.data[i]);
        flag = true;
      }
    break;

    case DOUBLEARRAY:
      if(m_darrinIn.isNew()){
        m_darrinIn.read();
        delement = m_darrin.data.length();
        if(delement < extra){
          std::cout << "DataLength ERROR." << std::endl;
          return RTC::RTC_ERROR;
        }
        for(int i=0; i<delement; i++)
          darrnum.push_back(m_darrin.data[i]);
        flag = true;
      }
    break;

    case BOOLARRAY:
      if(m_barrinIn.isNew()){
        m_barrinIn.read();
        belement = m_barrin.data.length();
        if(belement < extra){
          std::cout << "DataLength ERROR." << std::endl;
          return RTC::RTC_ERROR;
        }
        for(int i=0; i<belement; i++)
          barrnum.push_back(m_barrin.data[i]);
        flag = true;
      }
    break;

    default:
      std::cout << "intype : " << intype << "  outtype : " << outtype << std::endl;
      return RTC::RTC_ERROR;
    break;
  }

  if(flag){
    switch(intype){
      case SHORT:
        m_shortout.data = shortnum;
        m_shortoutOut.write();
        m_longout.data = (long)shortnum;
        m_longoutOut.write();
        m_floatout.data = (float)shortnum;
        m_floatoutOut.write();
        m_doubleout.data = (double)shortnum;
        m_doubleoutOut.write();
        m_boolout.data = (bool)shortnum;
        m_booloutOut.write();

        m_sarrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_sarrout.data[i] = shortnum;
        m_sarroutOut.write();
        m_larrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_larrout.data[i] = (long)shortnum;
        m_larroutOut.write();
        m_farrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_farrout.data[i] = (float)shortnum;
        m_farroutOut.write();
        m_darrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_darrout.data[i] = (double)shortnum;
        m_darroutOut.write();
        m_barrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_barrout.data[i] = (bool)shortnum;
        m_barroutOut.write();
      break;

      case LONG:
        m_shortout.data = (short)longnum;
        m_shortoutOut.write();
        m_longout.data = longnum;
        m_longoutOut.write();
        m_floatout.data = (float)longnum;
        m_floatoutOut.write();
        m_doubleout.data = (double)longnum;
        m_doubleoutOut.write();
        m_boolout.data = (bool)longnum;
        m_booloutOut.write();

        m_sarrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_sarrout.data[i] = (short)longnum;
        m_sarroutOut.write();
        m_larrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_larrout.data[i] = longnum;
        m_larroutOut.write();
        m_farrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_farrout.data[i] = (float)longnum;
        m_farroutOut.write();
        m_darrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_darrout.data[i] = (double)longnum;
        m_darroutOut.write();
        m_barrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_barrout.data[i] = (bool)longnum;
        m_barroutOut.write();
      break;

      case FLOAT:
        m_shortout.data = (short)floatnum;
        m_shortoutOut.write();
        m_longout.data = (long)floatnum;
        m_longoutOut.write();
        m_floatout.data = floatnum;
        m_floatoutOut.write();
        m_doubleout.data = (double)floatnum;
        m_doubleoutOut.write();
        m_boolout.data = (bool)floatnum;
        m_booloutOut.write();

        m_sarrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_sarrout.data[i] = (short)floatnum;
        m_sarroutOut.write();
        m_larrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_larrout.data[i] = (long)floatnum;
        m_larroutOut.write();
        m_farrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_farrout.data[i] = floatnum;
        m_farroutOut.write();
        m_darrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_darrout.data[i] = (double)floatnum;
        m_darroutOut.write();
        m_barrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_barrout.data[i] = (bool)floatnum;
        m_barroutOut.write();
      break;

      case DOUBLE:
        m_shortout.data = (short)doublenum;
        m_shortoutOut.write();
        m_longout.data = (long)doublenum;
        m_longoutOut.write();
        m_floatout.data = (float)doublenum;
        m_floatoutOut.write();
        m_doubleout.data = doublenum;
        m_doubleoutOut.write();
        m_boolout.data = (bool)doublenum;
        m_booloutOut.write();

        m_sarrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_sarrout.data[i] = (short)doublenum;
        m_sarroutOut.write();
        m_larrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_larrout.data[i] = (long)doublenum;
        m_larroutOut.write();
        m_farrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_farrout.data[i] = (float)doublenum;
        m_farroutOut.write();
        m_darrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_darrout.data[i] = doublenum;
        m_darroutOut.write();
        m_barrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_barrout.data[i] = (bool)doublenum;
        m_barroutOut.write();
      break;

      case BOOL:
        m_shortout.data = (short)boolnum;
        m_shortoutOut.write();
        m_longout.data = (long)boolnum;
        m_longoutOut.write();
        m_floatout.data = (float)boolnum;
        m_floatoutOut.write();
        m_doubleout.data = (double)boolnum;
        m_doubleoutOut.write();
        m_boolout.data = boolnum;
        m_booloutOut.write();

        m_sarrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_sarrout.data[i] = (short)boolnum;
        m_sarroutOut.write();
        m_larrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_larrout.data[i] = (long)boolnum;
        m_larroutOut.write();
        m_farrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_farrout.data[i] = (float)boolnum;
        m_farroutOut.write();
        m_darrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_darrout.data[i] = (double)boolnum;
        m_darroutOut.write();
        m_barrout.data.length(extra);
        for(int i=0; i<extra; i++)
          m_barrout.data[i] = boolnum;
        m_barroutOut.write();
      break;

      case SHORTARRAY:
        m_shortout.data = sarrnum[extra-1];
        m_shortoutOut.write();
        m_longout.data = (long)sarrnum[extra-1];
        m_longoutOut.write();
        m_floatout.data = (float)sarrnum[extra-1];
        m_floatoutOut.write();
        m_doubleout.data = (double)sarrnum[extra-1];
        m_doubleoutOut.write();
        m_boolout.data = (bool)sarrnum[extra-1];
        m_booloutOut.write();

        m_sarrout.data.length(selement);
        for(int i=0; i<selement; i++)
          m_sarrout.data[i] = sarrnum[i];
        m_sarroutOut.write();
        m_larrout.data.length(selement);
        for(int i=0; i<selement; i++)
          m_larrout.data[i] = (long)sarrnum[i];
        m_larroutOut.write();
        m_farrout.data.length(selement);
        for(int i=0; i<selement; i++)
          m_farrout.data[i] = (float)sarrnum[i];
        m_farroutOut.write();
        m_darrout.data.length(selement);
        for(int i=0; i<selement; i++)
          m_darrout.data[i] = (double)sarrnum[i];
        m_darroutOut.write();
        m_barrout.data.length(selement);
        for(int i=0; i<selement; i++)
          m_barrout.data[i] = (bool)sarrnum[i];
        m_barroutOut.write();
      break;

      case LONGARRAY:
        m_shortout.data = (short)larrnum[extra-1];
        m_shortoutOut.write();
        m_longout.data = larrnum[extra-1];
        m_longoutOut.write();
        m_floatout.data = (float)larrnum[extra-1];
        m_floatoutOut.write();
        m_doubleout.data = (double)larrnum[extra-1];
        m_doubleoutOut.write();
        m_boolout.data = (bool)larrnum[extra-1];
        m_booloutOut.write();

        m_sarrout.data.length(lelement);
        for(int i=0; i<lelement; i++)
          m_sarrout.data[i] = (short)larrnum[i];
        m_sarroutOut.write();
        m_larrout.data.length(lelement);
        for(int i=0; i<lelement; i++)
          m_larrout.data[i] = larrnum[i];
        m_larroutOut.write();
        m_farrout.data.length(lelement);
        for(int i=0; i<lelement; i++)
          m_farrout.data[i] = (float)larrnum[i];
        m_farroutOut.write();
        m_darrout.data.length(lelement);
        for(int i=0; i<lelement; i++)
          m_darrout.data[i] = (double)larrnum[i];
        m_darroutOut.write();
        m_barrout.data.length(lelement);
        for(int i=0; i<lelement; i++)
          m_barrout.data[i] = (bool)larrnum[i];
        m_barroutOut.write();
      break;

      case FLOATARRAY:
        m_shortout.data = (short)farrnum[extra-1];
        m_shortoutOut.write();
        m_longout.data = (long)farrnum[extra-1];
        m_longoutOut.write();
        m_floatout.data = farrnum[extra-1];
        m_floatoutOut.write();
        m_doubleout.data = (double)farrnum[extra-1];
        m_doubleoutOut.write();
        m_boolout.data = (bool)farrnum[extra-1];
        m_booloutOut.write();

        m_sarrout.data.length(felement);
        for(int i=0; i<felement; i++)
          m_sarrout.data[i] = (short)farrnum[i];
        m_sarroutOut.write();
        m_larrout.data.length(felement);
        for(int i=0; i<felement; i++)
          m_larrout.data[i] = (long)farrnum[i];
        m_larroutOut.write();
        m_farrout.data.length(felement);
        for(int i=0; i<felement; i++)
          m_farrout.data[i] = farrnum[i];
        m_farroutOut.write();
        m_darrout.data.length(felement);
        for(int i=0; i<felement; i++)
          m_darrout.data[i] = (double)farrnum[i];
        m_darroutOut.write();
        m_barrout.data.length(felement);
        for(int i=0; i<felement; i++)
          m_barrout.data[i] = (bool)farrnum[i];
        m_barroutOut.write();
      break;

      case DOUBLEARRAY:
        m_shortout.data = (short)darrnum[extra-1];
        m_shortoutOut.write();
        m_longout.data = (long)darrnum[extra-1];
        m_longoutOut.write();
        m_floatout.data = (float)darrnum[extra-1];
        m_floatoutOut.write();
        m_doubleout.data = darrnum[extra-1];
        m_doubleoutOut.write();
        m_boolout.data = (bool)darrnum[extra-1];
        m_booloutOut.write();

        m_sarrout.data.length(delement);
        for(int i=0; i<delement; i++)
          m_sarrout.data[i] = (short)darrnum[i];
        m_sarroutOut.write();
        m_larrout.data.length(delement);
        for(int i=0; i<delement; i++)
          m_larrout.data[i] = (long)darrnum[i];
        m_larroutOut.write();
        m_farrout.data.length(delement);
        for(int i=0; i<delement; i++)
          m_farrout.data[i] = (float)darrnum[i];
        m_farroutOut.write();
        m_darrout.data.length(delement);
        for(int i=0; i<delement; i++)
          m_darrout.data[i] = darrnum[i];
        m_darroutOut.write();
        m_barrout.data.length(delement);
        for(int i=0; i<delement; i++)
          m_barrout.data[i] = (bool)darrnum[i];
        m_barroutOut.write();
      break;

      case BOOLARRAY:
        m_shortout.data = (short)barrnum[extra-1];
        m_shortoutOut.write();
        m_longout.data = (long)barrnum[extra-1];
        m_longoutOut.write();
        m_floatout.data = (float)barrnum[extra-1];
        m_floatoutOut.write();
        m_doubleout.data = (double)barrnum[extra-1];
        m_doubleoutOut.write();
        m_boolout.data = barrnum[extra-1];
        m_booloutOut.write();

        m_sarrout.data.length(belement);
        for(int i=0; i<belement; i++)
          m_sarrout.data[i] = (short)barrnum[i];
        m_sarroutOut.write();
        m_larrout.data.length(belement);
        for(int i=0; i<belement; i++)
          m_larrout.data[i] = (long)barrnum[i];
        m_larroutOut.write();
        m_farrout.data.length(belement);
        for(int i=0; i<belement; i++)
          m_farrout.data[i] = (float)barrnum[i];
        m_farroutOut.write();
        m_darrout.data.length(belement);
        for(int i=0; i<belement; i++)
          m_darrout.data[i] = (double)barrnum[i];
        m_darroutOut.write();
        m_barrout.data.length(belement);
        for(int i=0; i<belement; i++)
          m_barrout.data[i] = barrnum[i];
        m_barroutOut.write();
      break;
    }
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Converter::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Converter::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Converter::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Converter::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Converter::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ConverterInit(RTC::Manager* manager)
  {
    RTC::Properties profile(converter_spec);
    manager->registerFactory(profile,
                             RTC::Create<Converter>,
                             RTC::Delete<Converter>);
  }
  
};


