// -*- C++ -*-
/*!
 * @file  FileReader.cpp
 * @brief FileReader component
 * $Date$
 *
 * $Id$
 */

#include "FileReader.h"
#include <fstream>
#include <iostream>
#include <iomanip>
#include <vector>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40

// Module specification
// <rtc-template block="module_spec">
static const char* filereader_spec[] =
  {
    "implementation_id", "FileReader",
    "type_name",         "FileReader",
    "description",       "FileReader component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    //"conf.default.OutputData", "sin.dat",
    "conf.default.FileName", "sin.dat",
    "conf.default.Start_Flag", "0",
    "conf.default.Array_Size", "1",

    ""
  };
// </rtc-template>

FileReader::FileReader(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortoutOut("shortout", m_shortout),
    m_longoutOut("longout", m_longout),
    m_floatoutOut("floatout", m_floatout),
    m_doubleoutOut("doubleout", m_doubleout),

    m_sarroutOut("shortarrayout", m_sarrout),
    m_larroutOut("longarrayout", m_larrout),
    m_farroutOut("floatarrayout", m_farrout),
    m_darroutOut("doublearrayout", m_darrout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("filereader", "filereader", m_filereader);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

FileReader::~FileReader()
{
}


RTC::ReturnCode_t FileReader::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("FileName", m_OutputData, "sin.dat");
  bindParameter("Start_Flag", m_Start_Flag, "0");
  bindParameter("Array_Size", m_Array_Size, "1");
  
  // </rtc-template>
  std::cout << "FileReader start." << std::endl;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileReader::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}


bool svc_flag=false;
bool svcname_flag=false;
int svc_type=0;
std::string svc_outputdata="0";
int svc_arraysize=0;
bool execute_flag=false;
bool svc_reset=false;

bool activate_flag=true;

std::vector<double> datayoko;
std::vector<std::vector<double> > datatate;
bool fill_flag=false;
std::vector<bool> data_flag;


RTC::ReturnCode_t FileReader::onActivated(RTC::UniqueId ec_id)
{
  std::ifstream fin;
  int dataatama;
  std::string dummyatama;
  std::streampos original;     //現在の位置の保存

  if(activate_flag){
    if(svc_flag)
      type = svc_type;
    else{
      if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="shortArray" && m_DataType!="longArray" && m_DataType!="floatArray" && m_DataType!="doubleArray"){
        std::cout << "DataType ERROR" << std::endl;
        return RTC::RTC_ERROR;
      }
      if(m_DataType=="short")
        type = SHORT;
      else if(m_DataType=="long")
        type = LONG;
      else if(m_DataType=="float")
        type = FLOAT;
      else if(m_DataType=="double")
        type = DOUBLE;
      else if(m_DataType=="shortArray")
        type = SHORTARRAY;
      else if(m_DataType=="longArray")
        type = LONGARRAY;
      else if(m_DataType=="floatArray")
        type = FLOATARRAY;
      else if(m_DataType=="doubleArray")
        type = DOUBLEARRAY;
    }

    switch(type){
      case SHORT:
        registerOutPort("shortout", m_shortoutOut);
      break;
      case LONG:
        registerOutPort("longout", m_longoutOut);
      break;
      case FLOAT:
        registerOutPort("floatout", m_floatoutOut);
      break;
      case DOUBLE:
        registerOutPort("doubleout", m_doubleoutOut);
      break;
      case SHORTARRAY:
        registerOutPort("shortArrayout", m_sarroutOut);
      break;
      case LONGARRAY:
        registerOutPort("longArrayout", m_larroutOut);
      break;
      case FLOATARRAY:
        registerOutPort("floatArrayout", m_farroutOut);
      break;
      case DOUBLEARRAY:
        registerOutPort("doubleArrayout", m_darroutOut);
      break;
    }

    if(svc_flag)
      arraysize = svc_arraysize;
    else
      arraysize = m_Array_Size;

    if(svcname_flag)
      outputdata = svc_outputdata;
    else
      outputdata = m_OutputData;

    std::cout << outputdata << " will open." << std::endl;

    fin.open(outputdata.c_str()); //ファイル読み込み
    if(!fin){
      std::cout << "file can't open!" << std::endl;
      return RTC::RTC_ERROR;
    }

    for(int i=0; i<arraysize; i++) //データの要素分だけ配列を広げる
      datayoko.push_back(0);

    fin >> dummyatama;     //fill_flag確認
    if(dummyatama == "FILL")
      fill_flag = true;
    else
      fin.seekg(0, std::ios::beg);

    for(int i=0; ; i++){ //文字列の終了判断
      original = fin.tellg();
      if(!(getline(fin, dummyatama))) //１回だけだとEOF的な文字まで読むっぽい.EOFの認識方法不明
        break;
      else{
        if(!(getline(fin, dummyatama)))
          break;
      }
      fin.seekg(original);  

      fin >> dataatama;
      for(; i!=dataatama; i++){ //dataatamaとiが一致するまで配列を増やす
        datatate.push_back(datayoko);
        data_flag.push_back(false);
      }
      for(int j=0; j<arraysize; j++)
        fin >> datayoko[j];
      datatate.push_back(datayoko);
      data_flag.push_back(true);
    }

    std::cout << "File_Read finish." << std::endl;
    fin.close();

    activate_flag=false;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileReader::onDeactivated(RTC::UniqueId ec_id)
{
  //datatate.clear();
  //datayoko.clear();
  //data_flag.clear();
  comptime=0;

  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileReader::onExecute(RTC::UniqueId ec_id)
{
  bool start_flag=false;

  if(svc_flag)
    start_flag = execute_flag;
  else
    start_flag = m_Start_Flag;

  if(start_flag){
    switch(type){
      case SHORT:
        if(fill_flag || data_flag[comptime]){
          m_shortout.data = (short)datatate[comptime][0];
          m_shortoutOut.write();
        }
      break;

      case LONG:
        if(fill_flag || data_flag[comptime]){
          m_longout.data = (short)datatate[comptime][0];
          m_longoutOut.write();
        }
      break;

      case FLOAT:
        if(fill_flag || data_flag[comptime]){
          m_floatout.data = (float)datatate[comptime][0];
          m_floatoutOut.write();
        }
      break;

      case DOUBLE:
        if(fill_flag || data_flag[comptime]){
          m_doubleout.data = datatate[comptime][0];
          m_doubleoutOut.write();
        }
      break;

      case SHORTARRAY:
        if(fill_flag || data_flag[comptime]){
          m_sarrout.data.length(arraysize);
          for(int i=0; i<arraysize; i++)
            m_sarrout.data[i] = (short)datatate[comptime][i];
          m_sarroutOut.write();
        }
      break;

      case LONGARRAY:
        if(fill_flag || data_flag[comptime]){
          m_larrout.data.length(arraysize);
          for(int i=0; i<arraysize; i++)
            m_larrout.data[i] = (long)datatate[comptime][i];
          m_larroutOut.write();
        }
      break;

      case FLOATARRAY:
        if(fill_flag || data_flag[comptime]){
          m_farrout.data.length(arraysize);
          for(int i=0; i<arraysize; i++)
            m_farrout.data[i] = (float)datatate[comptime][i];
          m_farroutOut.write();
        }
      break;

      case DOUBLEARRAY:
        if(fill_flag || data_flag[comptime]){
          m_darrout.data.length(arraysize);
          for(int i=0; i<arraysize; i++)
            m_darrout.data[i] = datatate[comptime][i];
          m_darroutOut.write();
        }
      break;
    }

    comptime++;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t FileReader::onStateUpdate(RTC::UniqueId ec_id)
{
  if((comptime >= datatate.size()) || svc_reset){
    comptime = 0;
    svc_reset=false;
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t FileReader::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FileReader::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FileReader::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FileReader::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void FileReaderInit(RTC::Manager* manager)
  {
    RTC::Properties profile(filereader_spec);
    manager->registerFactory(profile,
                             RTC::Create<FileReader>,
                             RTC::Delete<FileReader>);
  }
  
};


