// -*- C++ -*-
/*!
 * @file  Gainer.cpp
 * @brief Gainer component
 * $Date$
 *
 * $Id$
 */

#include "Gainer.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40

// Module specification
// <rtc-template block="module_spec">
static const char* gainer_spec[] =
  {
    "implementation_id", "Gainer",
    "type_name",         "Gainer",
    "description",       "Gainer component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.Gain", "1.0",

    ""
  };
// </rtc-template>

Gainer::Gainer(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortinIn("shortin", m_shortin),
    m_longinIn("longin", m_longin),
    m_floatinIn("floatin", m_floatin),
    m_doubleinIn("doublein", m_doublein),

    m_sarrinIn("shortarrayin", m_sarrin),
    m_larrinIn("longarrayin", m_larrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),

    m_shortoutOut("shortout", m_shortout),
    m_longoutOut("longout", m_longout),
    m_floatoutOut("floatout", m_floatout),
    m_doubleoutOut("doubleout", m_doubleout),

    m_sarroutOut("shortarrayout", m_sarrout),
    m_larroutOut("longarrayout", m_larrout),
    m_farroutOut("floatarrayout", m_farrout),
    m_darroutOut("doublearrayout", m_darrout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("gainer", "gainer", m_gainer);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Gainer::~Gainer()
{
}


RTC::ReturnCode_t Gainer::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("Gain", m_Gain, "1.0");
  
  // </rtc-template>
  std::cout << "Gainer start." << std::endl;
  return RTC::RTC_OK;
}




RTC::ReturnCode_t Gainer::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}



bool svc_flag=false;
int svc_type=0;
double svc_gain=0;
bool activate_flag=true;

RTC::ReturnCode_t Gainer::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    type = svc_type;
  else{
    if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="shortArray" && m_DataType!="longArray" && m_DataType!="floatArray" && m_DataType!="doubleArray"){
      std::cout << "DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_DataType=="short")
      type = SHORT;
    else if(m_DataType=="long")
      type = LONG;
    else if(m_DataType=="float")
      type = FLOAT;
    else if(m_DataType=="double")
      type = DOUBLE;
    else if(m_DataType=="shortArray")
      type = SHORTARRAY;
    else if(m_DataType=="longArray")
      type = LONGARRAY;
    else if(m_DataType=="floatArray")
      type = FLOATARRAY;
    else if(m_DataType=="doubleArray")
      type = DOUBLEARRAY;
  }


  if(svc_flag)
    gain = svc_gain;
  else
    gain = m_Gain;


  switch(type){
    case SHORT:
      registerInPort("shortin", m_shortinIn);
      registerOutPort("shortout", m_shortoutOut);
    break;
    case LONG:
      registerInPort("longin", m_longinIn);
      registerOutPort("longout", m_longoutOut);
    break;
    case FLOAT:
      registerInPort("floatin", m_floatinIn);
      registerOutPort("floatout", m_floatoutOut);
    break;
    case DOUBLE:
      registerInPort("doublein", m_doubleinIn);
      registerOutPort("doubleout", m_doubleoutOut);
    break;
    case SHORTARRAY:
      registerInPort("shortArrayin", m_sarrinIn);
      registerOutPort("shortArrayout", m_sarroutOut);
    break;
    case LONGARRAY:
      registerInPort("longArrayin", m_larrinIn);
      registerOutPort("longArrayout", m_larroutOut);
    break;
    case FLOATARRAY:
      registerInPort("floatArrayin", m_farrinIn);
      registerOutPort("floatArrayout", m_farroutOut);
    break;
    case DOUBLEARRAY:
      registerInPort("doubleArrayin", m_darrinIn);
      registerOutPort("doubleArrayout", m_darroutOut);
    break;
  }

  activate_flag=false;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Gainer::onDeactivated(RTC::UniqueId ec_id)
{
  m_shortinIn.read();
  m_longinIn.read();
  m_floatinIn.read();
  m_doubleinIn.read();
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Gainer::onExecute(RTC::UniqueId ec_id)
{
  bool flag=false;
  short shortnum = 0;
  long longnum = 0;
  float floatnum = 0;
  double doublenum = 0;
  int selement = 0, lelement = 0, felement = 0, delement = 0;
  std::vector<short> sarrnum;
  std::vector<long> larrnum;
  std::vector<float> farrnum;
  std::vector<double> darrnum;

  if(svc_flag)
    gain = svc_gain;
  else
    gain = m_Gain;

  if(m_shortinIn.isNew()){
    m_shortinIn.read();
    shortnum = (short)(gain * m_shortin.data);
    flag = true;
  }
  if(m_longinIn.isNew()){
    m_longinIn.read();
    longnum = (long)(gain * m_longin.data);
    flag = true;
  }
  if(m_floatinIn.isNew()){
    m_floatinIn.read();
    floatnum = (float)(gain * m_floatin.data);
    flag = true;
  }
  if(m_doubleinIn.isNew()){
    m_doubleinIn.read();
    doublenum = (double)(gain * m_doublein.data);
    flag = true;
  }
  if(m_sarrinIn.isNew()){
    m_sarrinIn.read();
    selement = m_sarrin.data.length();
    for(int i=0; i<selement; i++)
      sarrnum.push_back((short)(gain * m_sarrin.data[i]));
    flag = true;
  }
  if(m_larrinIn.isNew()){
    m_larrinIn.read();
    lelement = m_larrin.data.length();
    for(int i=0; i<lelement; i++)
      larrnum.push_back((long)(gain * m_larrin.data[i]));
    flag = true;
  }
  if(m_farrinIn.isNew()){
    m_farrinIn.read();
    felement = m_farrin.data.length();
    for(int i=0; i<felement; i++)
      farrnum.push_back((float)(gain * m_farrin.data[i]));
    flag = true;
  }
  if(m_darrinIn.isNew()){
    m_darrinIn.read();
    delement = m_darrin.data.length();
    for(int i=0; i<delement; i++)
      darrnum.push_back((double)(gain * m_darrin.data[i]));
    flag = true;
  }

  if(flag){
    m_shortout.data = shortnum;
    m_shortoutOut.write();
    m_longout.data = longnum;
    m_longoutOut.write();
    m_floatout.data = floatnum;
    m_floatoutOut.write();
    m_doubleout.data = doublenum;
    m_doubleoutOut.write();

    m_sarrout.data.length(selement);
    for(int i=0; i<selement; i++)
      m_sarrout.data[i] = sarrnum[i];
    m_sarroutOut.write();
    m_larrout.data.length(lelement);
    for(int i=0; i<lelement; i++)
      m_larrout.data[i] = larrnum[i];
    m_larroutOut.write();
    m_farrout.data.length(felement);
    for(int i=0; i<felement; i++)
      m_farrout.data[i] = farrnum[i];
    m_farroutOut.write();
    m_darrout.data.length(delement);
    for(int i=0; i<delement; i++)
      m_darrout.data[i] = darrnum[i];
    m_darroutOut.write();
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Gainer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Gainer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Gainer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/




extern "C"
{
 
  void GainerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(gainer_spec);
    manager->registerFactory(profile,
                             RTC::Create<Gainer>,
                             RTC::Delete<Gainer>);
  }
  
};


