// -*- C++ -*-
/*!
 * @file  Limiter.cpp
 * @brief Limiter component
 * $Date$
 *
 * $Id$
 */

#include "Limiter.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define SHORTARRAY 10
#define LONGARRAY 20
#define FLOATARRAY 30
#define DOUBLEARRAY 40

// Module specification
// <rtc-template block="module_spec">
static const char* limiter_spec[] =
  {
    "implementation_id", "Limiter",
    "type_name",         "Limiter",
    "description",       "Limiter component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.Limit_MAX", "100",
    "conf.default.Limit_MIN", "-100",

    ""
  };
// </rtc-template>

Limiter::Limiter(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortinIn("shortin", m_shortin),
    m_longinIn("longin", m_longin),
    m_floatinIn("floatin", m_floatin),
    m_doubleinIn("doublein", m_doublein),

    m_sarrinIn("shortarrayin", m_sarrin),
    m_larrinIn("longarrayin", m_larrin),
    m_farrinIn("floatarrayin", m_farrin),
    m_darrinIn("doublearrayin", m_darrin),

    m_shortoutOut("shortout", m_shortout),
    m_longoutOut("longout", m_longout),
    m_floatoutOut("floatout", m_floatout),
    m_doubleoutOut("doubleout", m_doubleout),

    m_sarroutOut("shortarrayout", m_sarrout),
    m_larroutOut("longarrayout", m_larrout),
    m_farroutOut("floatarrayout", m_farrout),
    m_darroutOut("doublearrayout", m_darrout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("limiter", "limiter", m_limiter);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Limiter::~Limiter()
{
}


RTC::ReturnCode_t Limiter::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("Limit_MAX", m_Limit_MAX, "100");
  bindParameter("Limit_MIN", m_Limit_MIN, "-100");
  
  // </rtc-template>
  std::cout << "Limiter start." << std::endl;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t Limiter::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}


bool svc_flag=false;
int svc_type=0;
double svc_max=0;
double svc_min=0;
bool activate_flag=true;

RTC::ReturnCode_t Limiter::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

    if(svc_flag)
      type = svc_type;
    else{
      if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="shortArray" && m_DataType!="longArray" && m_DataType!="floatArray" && m_DataType!="doubleArray"){
        std::cout << "DataType ERROR" << std::endl;
        return RTC::RTC_ERROR;
      }
      if(m_DataType=="short")
        type = SHORT;
      else if(m_DataType=="long")
        type = LONG;
      else if(m_DataType=="float")
        type = FLOAT;
      else if(m_DataType=="double")
        type = DOUBLE;
      else if(m_DataType=="shortArray")
        type = SHORTARRAY;
      else if(m_DataType=="longArray")
        type = LONGARRAY;
      else if(m_DataType=="floatArray")
        type = FLOATARRAY;
      else if(m_DataType=="doubleArray")
        type = DOUBLEARRAY;
    }

    if(svc_flag){
      limitmax = svc_max;
      limitmin = svc_min;
    }
    else{
      limitmax = m_Limit_MAX;
      svc_max  = m_Limit_MAX;
      limitmin = m_Limit_MIN;
      svc_min  = m_Limit_MIN;
    }


    switch(type){
      case SHORT:
        registerInPort("shortin", m_shortinIn);
        registerOutPort("shortout", m_shortoutOut);
      break;
      case LONG:
        registerInPort("longin", m_longinIn);
        registerOutPort("longout", m_longoutOut);
      break;
      case FLOAT:
        registerInPort("floatin", m_floatinIn);
        registerOutPort("floatout", m_floatoutOut);
      break;
      case DOUBLE:
        registerInPort("doublein", m_doubleinIn);
        registerOutPort("doubleout", m_doubleoutOut);
      break;
      case SHORTARRAY:
        registerInPort("shortArrayin", m_sarrinIn);
        registerOutPort("shortArrayout", m_sarroutOut);
      break;
      case LONGARRAY:
        registerInPort("longArrayin", m_larrinIn);
        registerOutPort("longArrayout", m_larroutOut);
      break;
      case FLOATARRAY:
        registerInPort("floatArrayin", m_farrinIn);
        registerOutPort("floatArrayout", m_farroutOut);
      break;
      case DOUBLEARRAY:
        registerInPort("doubleArrayin", m_darrinIn);
        registerOutPort("doubleArrayout", m_darroutOut);
      break;
    }

    activate_flag=false;
  }

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Limiter::onDeactivated(RTC::UniqueId ec_id)
{
  m_shortinIn.read();
  m_longinIn.read();
  m_floatinIn.read();
  m_doubleinIn.read();
  m_sarrinIn.read();
  m_larrinIn.read();
  m_farrinIn.read();
  m_darrinIn.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Limiter::onExecute(RTC::UniqueId ec_id)
{
  if(svc_flag){
    limitmax = svc_max;
    limitmin = svc_min;
  }
  else{
    limitmax = m_Limit_MAX;
    svc_max  = m_Limit_MAX;
    limitmin = m_Limit_MIN;
    svc_min  = m_Limit_MIN;
  }

  switch(type){
    case SHORT:
      if(m_shortinIn.isNew()){
        m_shortinIn.read();
        if(m_shortin.data > limitmax)
          m_shortout.data = (short)limitmax;
        else if(m_shortin.data < limitmin)
          m_shortout.data = (short)limitmin;
        else
          m_shortout.data = m_shortin.data;
        m_shortoutOut.write();
      }
    break;

    case LONG:
      if(m_longinIn.isNew()){
        m_longinIn.read();
        if(m_longin.data > limitmax)
          m_longout.data = (long)limitmax;
        else if(m_longin.data < limitmin)
          m_longout.data = (long)limitmin;
        else
          m_longout.data = m_longin.data;
        m_longoutOut.write();
      }
    break;

    case FLOAT:
      if(m_floatinIn.isNew()){
        m_floatinIn.read();
        if(m_floatin.data > limitmax)
          m_floatout.data = (float)limitmax;
        else if(m_floatin.data < limitmin)
          m_floatout.data = (float)limitmin;
        else
          m_floatout.data = m_floatin.data;
        m_floatoutOut.write();
      }
    break;

    case DOUBLE:
      if(m_doubleinIn.isNew()){
        m_doubleinIn.read();
        if(m_doublein.data > limitmax)
          m_doubleout.data = (double)limitmax;
        else if(m_doublein.data < limitmin)
          m_doubleout.data = (double)limitmin;
        else
          m_doubleout.data = m_doublein.data;
        m_doubleoutOut.write();
      }
    break;

    case SHORTARRAY:
      if(m_sarrinIn.isNew()){
        m_sarrinIn.read();
        for(int i=0, len=m_sarrin.data.length(); i<len; i++){
          if(m_sarrin.data[i] > limitmax)
            m_sarrout.data[i] = (short)limitmax;
          else if(m_sarrin.data[i] < limitmin)
            m_sarrout.data[i] = (short)limitmin;
          else
            m_sarrout.data[i] = m_sarrin.data[i];
        }
        m_sarroutOut.write();
      }
    break;

    case LONGARRAY:
      if(m_larrinIn.isNew()){
        m_larrinIn.read();
        for(int i=0, len=m_larrin.data.length(); i<len; i++){
          if(m_larrin.data[i] > limitmax)
            m_larrout.data[i] = (long)limitmax;
          else if(m_larrin.data[i] < limitmin)
            m_larrout.data[i] = (long)limitmin;
          else
            m_larrout.data[i] = m_larrin.data[i];
        }
        m_larroutOut.write();
      }
    break;

    case FLOATARRAY:
      if(m_farrinIn.isNew()){
        m_farrinIn.read();
        for(int i=0, len=m_farrin.data.length(); i<len; i++){
          if(m_farrin.data[i] > limitmax)
            m_farrout.data[i] = (short)limitmax;
          else if(m_farrin.data[i] < limitmin)
            m_farrout.data[i] = (short)limitmin;
          else
            m_farrout.data[i] = m_farrin.data[i];
        }
        m_farroutOut.write();
      }
    break;

    case DOUBLEARRAY:
      if(m_darrinIn.isNew()){
        m_darrinIn.read();
        for(int i=0, len=m_darrin.data.length(); i<len; i++){
          if(m_darrin.data[i] > limitmax)
            m_darrout.data[i] = limitmax;
          else if(m_darrin.data[i] < limitmin)
            m_darrout.data[i] = limitmin;
          else
            m_darrout.data[i] = m_darrin.data[i];
        }
        m_darroutOut.write();
      }
    break;
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Limiter::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Limiter::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Limiter::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Limiter::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Limiter::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void LimiterInit(RTC::Manager* manager)
  {
    RTC::Properties profile(limiter_spec);
    manager->registerFactory(profile,
                             RTC::Create<Limiter>,
                             RTC::Delete<Limiter>);
  }
  
};


