// -*- C++ -*-
/*!
 * @file  Multiplexer.cpp
 * @brief Multiplexer component
 * $Date$
 *
 * $Id$
 */

#include "Multiplexer.h"
#include <iostream>

#define SHORT 1
#define LONG 2
#define FLOAT 3
#define DOUBLE 4
#define BOOL 5

// Module specification
// <rtc-template block="module_spec">
static const char* multiplexer_spec[] =
  {
    "implementation_id", "Multiplexer",
    "type_name",         "Multiplexer",
    "description",       "Multiplexer component",
    "version",           "1.0",
    "vendor",            "MS_LAB",
    "category",          "",
    "activity_type",     "",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.DataType", "double",
    "conf.default.InPort_Num", "3",

    ""
  };
// </rtc-template>

Multiplexer::Multiplexer(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_shortin0In("shortin0", m_shortin0),
    m_shortin1In("shortin1", m_shortin1),
    m_shortin2In("shortin2", m_shortin2),
    m_shortin3In("shortin3", m_shortin3),
    m_shortin4In("shortin4", m_shortin4),
    m_shortin5In("shortin5", m_shortin5),
    m_shortin6In("shortin6", m_shortin6),
    m_shortin7In("shortin7", m_shortin7),
    m_shortin8In("shortin8", m_shortin8),
    m_shortin9In("shortin9", m_shortin9),
    m_longin0In("longin0", m_longin0),
    m_longin1In("longin1", m_longin1),
    m_longin2In("longin2", m_longin2),
    m_longin3In("longin3", m_longin3),
    m_longin4In("longin4", m_longin4),
    m_longin5In("longin5", m_longin5),
    m_longin6In("longin6", m_longin6),
    m_longin7In("longin7", m_longin7),
    m_longin8In("longin8", m_longin8),
    m_longin9In("longin9", m_longin9),
    m_floatin0In("floatin0", m_floatin0),
    m_floatin1In("floatin1", m_floatin1),
    m_floatin2In("floatin2", m_floatin2),
    m_floatin3In("floatin3", m_floatin3),
    m_floatin4In("floatin4", m_floatin4),
    m_floatin5In("floatin5", m_floatin5),
    m_floatin6In("floatin6", m_floatin6),
    m_floatin7In("floatin7", m_floatin7),
    m_floatin8In("floatin8", m_floatin8),
    m_floatin9In("floatin9", m_floatin9),
    m_doublein0In("doublein0", m_doublein0),
    m_doublein1In("doublein1", m_doublein1),
    m_doublein2In("doublein2", m_doublein2),
    m_doublein3In("doublein3", m_doublein3),
    m_doublein4In("doublein4", m_doublein4),
    m_doublein5In("doublein5", m_doublein5),
    m_doublein6In("doublein6", m_doublein6),
    m_doublein7In("doublein7", m_doublein7),
    m_doublein8In("doublein8", m_doublein8),
    m_doublein9In("doublein9", m_doublein9),
    m_boolin0In("boolin0", m_boolin0),
    m_boolin1In("boolin1", m_boolin1),
    m_boolin2In("boolin2", m_boolin2),
    m_boolin3In("boolin3", m_boolin3),
    m_boolin4In("boolin4", m_boolin4),
    m_boolin5In("boolin5", m_boolin5),
    m_boolin6In("boolin6", m_boolin6),
    m_boolin7In("boolin7", m_boolin7),
    m_boolin8In("boolin8", m_boolin8),
    m_boolin9In("boolin9", m_boolin9),

    m_sarroutOut("shortarrayout", m_sarrout),
    m_larroutOut("longarrayout", m_larrout),
    m_farroutOut("floatarrayout", m_farrout),
    m_darroutOut("doublearrayout", m_darrout),
    m_barroutOut("booleanarrayout", m_barrout),

    m_PrvPortPort("PrvPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("datain", m_datainIn);
  
  // Set OutPort buffer
  //registerOutPort("dataout", m_dataoutOut);
  
  // Set service provider to Ports
  m_PrvPortPort.registerProvider("multiplexer", "multiplexer", m_multiplexer);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PrvPortPort);
  
  // </rtc-template>

}

Multiplexer::~Multiplexer()
{
}


RTC::ReturnCode_t Multiplexer::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("DataType", m_DataType, "double");
  bindParameter("InPort_Num", m_InPort_Num, "3");
  
  // </rtc-template>
  std::cout << "Multiplexer start." << std::endl;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t Multiplexer::onFinalize()
{
  RTC::Manager::instance().terminate();
  return RTC::RTC_OK;
}



bool svc_flag=false;
int svc_type=0;
int svc_num=0;
bool activate_flag=true;

RTC::ReturnCode_t Multiplexer::onActivated(RTC::UniqueId ec_id)
{
  if(activate_flag){

  if(svc_flag)
    type = svc_type;
  else{
    if(m_DataType!="short" && m_DataType!="long" && m_DataType!="float" && m_DataType!="double" && m_DataType!="bool"){
      std::cout << "DataType ERROR" << std::endl;
      return RTC::RTC_ERROR;
    }
    if(m_DataType=="short")
      type = SHORT;
    else if(m_DataType=="long")
      type = LONG;
    else if(m_DataType=="float")
      type = FLOAT;
    else if(m_DataType=="double")
      type = DOUBLE;
    else if(m_DataType=="bool")
      type = BOOL;
  }


  if(svc_flag)
    num = svc_num;
  else
    num = m_InPort_Num;

  if(num < 1 || 10 < num){
    std::cout << "InPort_Num ERROR" << std::endl;
    return RTC::RTC_ERROR;
  }


  switch(type){
    case SHORT:
      registerOutPort("shortArrayout", m_sarroutOut);
      if(num > 0)
        registerInPort("shortin0", m_shortin0In);
      if(num > 1)
        registerInPort("shortin1", m_shortin1In);
      if(num > 2)
        registerInPort("shortin2", m_shortin2In);
      if(num > 3)
        registerInPort("shortin3", m_shortin3In);
      if(num > 4)
        registerInPort("shortin4", m_shortin4In);
      if(num > 5)
        registerInPort("shortin5", m_shortin5In);
      if(num > 6)
        registerInPort("shortin6", m_shortin6In);
      if(num > 7)
        registerInPort("shortin7", m_shortin7In);
      if(num > 8)
        registerInPort("shortin8", m_shortin8In);
      if(num > 9)
        registerInPort("shortin9", m_shortin9In);
    break;

    case LONG:
      registerOutPort("longArrayout", m_larroutOut);
      if(num > 0)
        registerInPort("longin0", m_longin0In);
      if(num > 1)
        registerInPort("longin1", m_longin1In);
      if(num > 2)
        registerInPort("longin2", m_longin2In);
      if(num > 3)
        registerInPort("longin3", m_longin3In);
      if(num > 4)
        registerInPort("longin4", m_longin4In);
      if(num > 5)
        registerInPort("longin5", m_longin5In);
      if(num > 6)
        registerInPort("longin6", m_longin6In);
      if(num > 7)
        registerInPort("longin7", m_longin7In);
      if(num > 8)
        registerInPort("longin8", m_longin8In);
      if(num > 9)
        registerInPort("longin9", m_longin9In);
    break;

    case FLOAT:
      registerOutPort("floatArrayout", m_farroutOut);
      if(num > 0)
        registerInPort("floatin0", m_floatin0In);
      if(num > 1)
        registerInPort("floatin1", m_floatin1In);
      if(num > 2)
        registerInPort("floatin2", m_floatin2In);
      if(num > 3)
        registerInPort("floatin3", m_floatin3In);
      if(num > 4)
        registerInPort("floatin4", m_floatin4In);
      if(num > 5)
        registerInPort("floatin5", m_floatin5In);
      if(num > 6)
        registerInPort("floatin6", m_floatin6In);
      if(num > 7)
        registerInPort("floatin7", m_floatin7In);
      if(num > 8)
        registerInPort("floatin8", m_floatin8In);
      if(num > 9)
        registerInPort("floatin9", m_floatin9In);
    break;

    case DOUBLE:
      registerOutPort("doubleArrayout", m_darroutOut);
      if(num > 0)
        registerInPort("doublein0", m_doublein0In);
      if(num > 1)
        registerInPort("doublein1", m_doublein1In);
      if(num > 2)
        registerInPort("doublein2", m_doublein2In);
      if(num > 3)
        registerInPort("doublein3", m_doublein3In);
      if(num > 4)
        registerInPort("doublein4", m_doublein4In);
      if(num > 5)
        registerInPort("doublein5", m_doublein5In);
      if(num > 6)
        registerInPort("doublein6", m_doublein6In);
      if(num > 7)
        registerInPort("doublein7", m_doublein7In);
      if(num > 8)
        registerInPort("doublein8", m_doublein8In);
      if(num > 9)
        registerInPort("doublein9", m_doublein9In);
    break;

    case BOOL:
      registerOutPort("booleanArrayout", m_barroutOut);
      if(num > 0)
        registerInPort("booleanin0", m_boolin0In);
      if(num > 1)
        registerInPort("booleanin1", m_boolin1In);
      if(num > 2)
        registerInPort("booleanin2", m_boolin2In);
      if(num > 3)
        registerInPort("booleanin3", m_boolin3In);
      if(num > 4)
        registerInPort("booleanin4", m_boolin4In);
      if(num > 5)
        registerInPort("booleanin5", m_boolin5In);
      if(num > 6)
        registerInPort("booleanin6", m_boolin6In);
      if(num > 7)
        registerInPort("booleanin7", m_boolin7In);
      if(num > 8)
        registerInPort("booleanin8", m_boolin8In);
      if(num > 9)
        registerInPort("booleanin9", m_boolin9In);
    break;
  }

  activate_flag=false;
  }

  return RTC::RTC_OK;
}


bool flag[10]={false, false, false, false, false, false, false, false, false, false};
short shortnum[10] = {0};
long longnum[10] = {0};
float floatnum[10] = {0};
double doublenum[10] = {0};
bool boolnum[10] = {false};

RTC::ReturnCode_t Multiplexer::onDeactivated(RTC::UniqueId ec_id)
{
  for(int i=0; i<10; i++){
    flag[i] = false;
    shortnum[i] = 0;
    longnum[i] = 0;
    floatnum[i] = 0;
    doublenum[i] = 0;
    boolnum[i] = false;
  }
  m_shortin0In.read();
  m_shortin1In.read();
  m_shortin2In.read();
  m_shortin3In.read();
  m_shortin4In.read();
  m_shortin5In.read();
  m_shortin6In.read();
  m_shortin7In.read();
  m_shortin8In.read();
  m_shortin9In.read();
  m_longin0In.read();
  m_longin1In.read();
  m_longin2In.read();
  m_longin3In.read();
  m_longin4In.read();
  m_longin5In.read();
  m_longin6In.read();
  m_longin7In.read();
  m_longin8In.read();
  m_longin9In.read();
  m_floatin0In.read();
  m_floatin1In.read();
  m_floatin2In.read();
  m_floatin3In.read();
  m_floatin4In.read();
  m_floatin5In.read();
  m_floatin6In.read();
  m_floatin7In.read();
  m_floatin8In.read();
  m_floatin9In.read();
  m_doublein0In.read();
  m_doublein1In.read();
  m_doublein2In.read();
  m_doublein3In.read();
  m_doublein4In.read();
  m_doublein5In.read();
  m_doublein6In.read();
  m_doublein7In.read();
  m_doublein8In.read();
  m_doublein9In.read();
  m_boolin0In.read();
  m_boolin1In.read();
  m_boolin2In.read();
  m_boolin3In.read();
  m_boolin4In.read();
  m_boolin5In.read();
  m_boolin6In.read();
  m_boolin7In.read();
  m_boolin8In.read();
  m_boolin9In.read();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Multiplexer::onExecute(RTC::UniqueId ec_id)
{
  bool all_read_flag=false;

  if(num >= 1){
    if(m_shortin0In.isNew()){
      m_shortin0In.read();
      shortnum[0] = m_shortin0.data;
      flag[0] = true;
    }
    if(m_longin0In.isNew()){
      m_longin0In.read();
      longnum[0] = m_longin0.data;
      flag[0] = true;
    }
    if(m_floatin0In.isNew()){
      m_floatin0In.read();
      floatnum[0] = m_floatin0.data;
      flag[0] = true;
    }
    if(m_doublein0In.isNew()){
      m_doublein0In.read();
      doublenum[0] = m_doublein0.data;
      flag[0] = true;
    }
    if(m_boolin0In.isNew()){
      m_boolin0In.read();
      boolnum[0] = m_boolin0.data;
      flag[0] = true;
    }
  }
  if(num >= 2){
    if(m_shortin1In.isNew()){
      m_shortin1In.read();
      shortnum[1] = m_shortin1.data;
      flag[1] = true;
    }
    if(m_longin1In.isNew()){
      m_longin1In.read();
      longnum[1] = m_longin1.data;
      flag[1] = true;
    }
    if(m_floatin1In.isNew()){
      m_floatin1In.read();
      floatnum[1] = m_floatin1.data;
      flag[1] = true;
    }
    if(m_doublein1In.isNew()){
      m_doublein1In.read();
      doublenum[1] = m_doublein1.data;
      flag[1] = true;
    }
    if(m_boolin1In.isNew()){
      m_boolin1In.read();
      boolnum[1] = m_boolin1.data;
      flag[1] = true;
    }
  }
  if(num >= 3){
    if(m_shortin2In.isNew()){
      m_shortin2In.read();
      shortnum[2] = m_shortin2.data;
      flag[2] = true;
    }
    if(m_longin2In.isNew()){
      m_longin2In.read();
      longnum[2] = m_longin2.data;
      flag[2] = true;
    }
    if(m_floatin2In.isNew()){
      m_floatin2In.read();
      floatnum[2] = m_floatin2.data;
      flag[2] = true;
    }
    if(m_doublein2In.isNew()){
      m_doublein2In.read();
      doublenum[2] = m_doublein2.data;
      flag[2] = true;
    }
    if(m_boolin2In.isNew()){
      m_boolin2In.read();
      boolnum[2] = m_boolin2.data;
      flag[2] = true;
    }
  }
  if(num >= 4){
    if(m_shortin3In.isNew()){
      m_shortin3In.read();
      shortnum[3] = m_shortin3.data;
      flag[3] = true;
    }
    if(m_longin3In.isNew()){
      m_longin3In.read();
      longnum[3] = m_longin3.data;
      flag[3] = true;
    }
    if(m_floatin3In.isNew()){
      m_floatin3In.read();
      floatnum[3] = m_floatin3.data;
      flag[3] = true;
    }
    if(m_doublein3In.isNew()){
      m_doublein3In.read();
      doublenum[3] = m_doublein3.data;
      flag[3] = true;
    }
    if(m_boolin3In.isNew()){
      m_boolin3In.read();
      boolnum[3] = m_boolin3.data;
      flag[3] = true;
    }
  }

  if(num >= 5){
    if(m_shortin4In.isNew()){
      m_shortin4In.read();
      shortnum[4] = m_shortin4.data;
      flag[4] = true;
    }
    if(m_longin4In.isNew()){
      m_longin4In.read();
      longnum[4] = m_longin4.data;
      flag[4] = true;
    }
    if(m_floatin4In.isNew()){
      m_floatin4In.read();
      floatnum[4] = m_floatin4.data;
      flag[4] = true;
    }
    if(m_doublein4In.isNew()){
      m_doublein4In.read();
      doublenum[4] = m_doublein4.data;
      flag[4] = true;
    }
    if(m_boolin4In.isNew()){
      m_boolin4In.read();
      boolnum[4] = m_boolin4.data;
      flag[4] = true;
    }
  }

  if(num >= 6){
    if(m_shortin5In.isNew()){
      m_shortin5In.read();
      shortnum[5] = m_shortin5.data;
      flag[5] = true;
    }
    if(m_longin5In.isNew()){
      m_longin5In.read();
      longnum[5] = m_longin5.data;
      flag[5] = true;
    }
    if(m_floatin5In.isNew()){
      m_floatin5In.read();
      floatnum[5] = m_floatin5.data;
      flag[5] = true;
    }
    if(m_doublein5In.isNew()){
      m_doublein5In.read();
      doublenum[5] = m_doublein5.data;
      flag[5] = true;
    }
    if(m_boolin5In.isNew()){
      m_boolin5In.read();
      boolnum[5] = m_boolin5.data;
      flag[5] = true;
    }
  }

  if(num >= 7){
    if(m_shortin6In.isNew()){
      m_shortin6In.read();
      shortnum[6] = m_shortin6.data;
      flag[6] = true;
    }
    if(m_longin6In.isNew()){
      m_longin6In.read();
      longnum[6] = m_longin6.data;
      flag[6] = true;
    }
    if(m_floatin6In.isNew()){
      m_floatin6In.read();
      floatnum[6] = m_floatin6.data;
      flag[6] = true;
    }
    if(m_doublein6In.isNew()){
      m_doublein6In.read();
      doublenum[6] = m_doublein6.data;
      flag[6] = true;
    }
    if(m_boolin6In.isNew()){
      m_boolin6In.read();
      boolnum[6] = m_boolin6.data;
      flag[6] = true;
    }
  }

  if(num >= 8){
    if(m_shortin7In.isNew()){
      m_shortin7In.read();
      shortnum[7] = m_shortin7.data;
      flag[7] = true;
    }
    if(m_longin7In.isNew()){
      m_longin7In.read();
      longnum[7] = m_longin7.data;
      flag[7] = true;
    }
    if(m_floatin7In.isNew()){
      m_floatin7In.read();
      floatnum[7] = m_floatin7.data;
      flag[7] = true;
    }
    if(m_doublein7In.isNew()){
      m_doublein7In.read();
      doublenum[7] = m_doublein7.data;
      flag[7] = true;
    }
    if(m_boolin7In.isNew()){
      m_boolin7In.read();
      boolnum[7] = m_boolin7.data;
      flag[7] = true;
    }
  }
  
  if(num >= 9){
    if(m_shortin8In.isNew()){
      m_shortin8In.read();
      shortnum[8] = m_shortin8.data;
      flag[8] = true;
    }
    if(m_longin8In.isNew()){
      m_longin8In.read();
      longnum[8] = m_longin8.data;
      flag[8] = true;
    }
    if(m_floatin8In.isNew()){
      m_floatin8In.read();
      floatnum[8] = m_floatin8.data;
      flag[8] = true;
    }
    if(m_doublein8In.isNew()){
      m_doublein8In.read();
      doublenum[8] = m_doublein8.data;
      flag[8] = true;
    }
    if(m_boolin8In.isNew()){
      m_boolin8In.read();
      boolnum[8] = m_boolin8.data;
      flag[8] = true;
    }
  }

  if(num >= 10){
    if(m_shortin9In.isNew()){
      m_shortin9In.read();
      shortnum[9] = m_shortin9.data;
      flag[9] = true;
    }
    if(m_longin9In.isNew()){
      m_longin9In.read();
      longnum[9] = m_longin9.data;
      flag[9] = true;
    }
    if(m_floatin9In.isNew()){
      m_floatin9In.read();
      floatnum[9] = m_floatin9.data;
      flag[9] = true;
    }
    if(m_doublein9In.isNew()){
      m_doublein9In.read();
      doublenum[9] = m_doublein9.data;
      flag[9] = true;
    }
    if(m_boolin9In.isNew()){
      m_boolin9In.read();
      boolnum[9] = m_boolin9.data;
      flag[9] = true;
    }
  }

  all_read_flag = true;
  for(int i=0; i<num; i++){
    if(flag[i] == false)
      all_read_flag = false;
  }

  if(all_read_flag == true){
    switch(type){
      case SHORT:
        m_sarrout.data.length(num);
        for(int i=0; i<num; i++)
          m_sarrout.data[i] = shortnum[i];
        m_sarroutOut.write();
      break;

      case LONG:
        m_larrout.data.length(num);
        for(int i=0; i<num; i++)
          m_larrout.data[i] = longnum[i];
        m_larroutOut.write();
      break;

      case FLOAT:
        m_farrout.data.length(num);
        for(int i=0; i<num; i++)
          m_farrout.data[i] = floatnum[i];
        m_farroutOut.write();
      break;
 
      case DOUBLE:
        m_darrout.data.length(num);
        for(int i=0; i<num; i++)
          m_darrout.data[i] = doublenum[i];
        m_darroutOut.write();
      break;

      case BOOL:
        m_barrout.data.length(num);
        for(int i=0; i<num; i++)
          m_barrout.data[i] = boolnum[i];
        m_barroutOut.write();
      break;
    }

    for(int i=0; i<10; i++)
      flag[i] = false;

  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Multiplexer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Multiplexer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Multiplexer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/




extern "C"
{
 
  void MultiplexerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(multiplexer_spec);
    manager->registerFactory(profile,
                             RTC::Create<Multiplexer>,
                             RTC::Delete<Multiplexer>);
  }
  
};


