// -*- C++ -*-
/*!
 * @file  Summater.h
 * @brief Summater component
 * @date  $Date$
 *
 * $Id$
 */

#ifndef SUMMATER_H
#define SUMMATER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "SummaterSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class Summater
  : public RTC::DataFlowComponentBase
{
 public:
  Summater(RTC::Manager* manager);
  ~Summater();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_DataType;
  int m_InPort_Num;
  int m_Plus_Num;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedShort m_shortin0;
  InPort<TimedShort> m_shortin0In;
  TimedShort m_shortin1;
  InPort<TimedShort> m_shortin1In;
  TimedShort m_shortin2;
  InPort<TimedShort> m_shortin2In;
  TimedShort m_shortin3;
  InPort<TimedShort> m_shortin3In;
  TimedShort m_shortin4;
  InPort<TimedShort> m_shortin4In;
  TimedShort m_shortin5;
  InPort<TimedShort> m_shortin5In;
  TimedShort m_shortin6;
  InPort<TimedShort> m_shortin6In;
  TimedShort m_shortin7;
  InPort<TimedShort> m_shortin7In;
  TimedShort m_shortin8;
  InPort<TimedShort> m_shortin8In;
  TimedShort m_shortin9;
  InPort<TimedShort> m_shortin9In;

  TimedLong m_longin0;
  InPort<TimedLong> m_longin0In;
  TimedLong m_longin1;
  InPort<TimedLong> m_longin1In;
  TimedLong m_longin2;
  InPort<TimedLong> m_longin2In;
  TimedLong m_longin3;
  InPort<TimedLong> m_longin3In;
  TimedLong m_longin4;
  InPort<TimedLong> m_longin4In;
  TimedLong m_longin5;
  InPort<TimedLong> m_longin5In;
  TimedLong m_longin6;
  InPort<TimedLong> m_longin6In;
  TimedLong m_longin7;
  InPort<TimedLong> m_longin7In;
  TimedLong m_longin8;
  InPort<TimedLong> m_longin8In;
  TimedLong m_longin9;
  InPort<TimedLong> m_longin9In;

  TimedFloat m_floatin0;
  InPort<TimedFloat> m_floatin0In;
  TimedFloat m_floatin1;
  InPort<TimedFloat> m_floatin1In;
  TimedFloat m_floatin2;
  InPort<TimedFloat> m_floatin2In;
  TimedFloat m_floatin3;
  InPort<TimedFloat> m_floatin3In;
  TimedFloat m_floatin4;
  InPort<TimedFloat> m_floatin4In;
  TimedFloat m_floatin5;
  InPort<TimedFloat> m_floatin5In;
  TimedFloat m_floatin6;
  InPort<TimedFloat> m_floatin6In;
  TimedFloat m_floatin7;
  InPort<TimedFloat> m_floatin7In;
  TimedFloat m_floatin8;
  InPort<TimedFloat> m_floatin8In;
  TimedFloat m_floatin9;
  InPort<TimedFloat> m_floatin9In;

  TimedDouble m_doublein0;
  InPort<TimedDouble> m_doublein0In;
  TimedDouble m_doublein1;
  InPort<TimedDouble> m_doublein1In;
  TimedDouble m_doublein2;
  InPort<TimedDouble> m_doublein2In;
  TimedDouble m_doublein3;
  InPort<TimedDouble> m_doublein3In;
  TimedDouble m_doublein4;
  InPort<TimedDouble> m_doublein4In;
  TimedDouble m_doublein5;
  InPort<TimedDouble> m_doublein5In;
  TimedDouble m_doublein6;
  InPort<TimedDouble> m_doublein6In;
  TimedDouble m_doublein7;
  InPort<TimedDouble> m_doublein7In;
  TimedDouble m_doublein8;
  InPort<TimedDouble> m_doublein8In;
  TimedDouble m_doublein9;
  InPort<TimedDouble> m_doublein9In;

  TimedShortSeq m_sarrin0;
  InPort<TimedShortSeq> m_sarrin0In;
  TimedShortSeq m_sarrin1;
  InPort<TimedShortSeq> m_sarrin1In;
  TimedShortSeq m_sarrin2;
  InPort<TimedShortSeq> m_sarrin2In;
  TimedShortSeq m_sarrin3;
  InPort<TimedShortSeq> m_sarrin3In;
  TimedShortSeq m_sarrin4;
  InPort<TimedShortSeq> m_sarrin4In;
  TimedShortSeq m_sarrin5;
  InPort<TimedShortSeq> m_sarrin5In;
  TimedShortSeq m_sarrin6;
  InPort<TimedShortSeq> m_sarrin6In;
  TimedShortSeq m_sarrin7;
  InPort<TimedShortSeq> m_sarrin7In;
  TimedShortSeq m_sarrin8;
  InPort<TimedShortSeq> m_sarrin8In;
  TimedShortSeq m_sarrin9;
  InPort<TimedShortSeq> m_sarrin9In;

  TimedLongSeq m_larrin0;
  InPort<TimedLongSeq> m_larrin0In;
  TimedLongSeq m_larrin1;
  InPort<TimedLongSeq> m_larrin1In;
  TimedLongSeq m_larrin2;
  InPort<TimedLongSeq> m_larrin2In;
  TimedLongSeq m_larrin3;
  InPort<TimedLongSeq> m_larrin3In;
  TimedLongSeq m_larrin4;
  InPort<TimedLongSeq> m_larrin4In;
  TimedLongSeq m_larrin5;
  InPort<TimedLongSeq> m_larrin5In;
  TimedLongSeq m_larrin6;
  InPort<TimedLongSeq> m_larrin6In;
  TimedLongSeq m_larrin7;
  InPort<TimedLongSeq> m_larrin7In;
  TimedLongSeq m_larrin8;
  InPort<TimedLongSeq> m_larrin8In;
  TimedLongSeq m_larrin9;
  InPort<TimedLongSeq> m_larrin9In;
  
  TimedFloatSeq m_farrin0;
  InPort<TimedFloatSeq> m_farrin0In;
  TimedFloatSeq m_farrin1;
  InPort<TimedFloatSeq> m_farrin1In;
  TimedFloatSeq m_farrin2;
  InPort<TimedFloatSeq> m_farrin2In;
  TimedFloatSeq m_farrin3;
  InPort<TimedFloatSeq> m_farrin3In;
  TimedFloatSeq m_farrin4;
  InPort<TimedFloatSeq> m_farrin4In;
  TimedFloatSeq m_farrin5;
  InPort<TimedFloatSeq> m_farrin5In;
  TimedFloatSeq m_farrin6;
  InPort<TimedFloatSeq> m_farrin6In;
  TimedFloatSeq m_farrin7;
  InPort<TimedFloatSeq> m_farrin7In;
  TimedFloatSeq m_farrin8;
  InPort<TimedFloatSeq> m_farrin8In;
  TimedFloatSeq m_farrin9;
  InPort<TimedFloatSeq> m_farrin9In;

  TimedDoubleSeq m_darrin0;
  InPort<TimedDoubleSeq> m_darrin0In;
  TimedDoubleSeq m_darrin1;
  InPort<TimedDoubleSeq> m_darrin1In;
  TimedDoubleSeq m_darrin2;
  InPort<TimedDoubleSeq> m_darrin2In;
  TimedDoubleSeq m_darrin3;
  InPort<TimedDoubleSeq> m_darrin3In;
  TimedDoubleSeq m_darrin4;
  InPort<TimedDoubleSeq> m_darrin4In;
  TimedDoubleSeq m_darrin5;
  InPort<TimedDoubleSeq> m_darrin5In;
  TimedDoubleSeq m_darrin6;
  InPort<TimedDoubleSeq> m_darrin6In;
  TimedDoubleSeq m_darrin7;
  InPort<TimedDoubleSeq> m_darrin7In;
  TimedDoubleSeq m_darrin8;
  InPort<TimedDoubleSeq> m_darrin8In;
  TimedDoubleSeq m_darrin9;
  InPort<TimedDoubleSeq> m_darrin9In;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedShort m_shortout;
  OutPort<TimedShort> m_shortoutOut;
  TimedLong m_longout;
  OutPort<TimedLong> m_longoutOut;
  TimedFloat m_floatout;
  OutPort<TimedFloat> m_floatoutOut;
  TimedDouble m_doubleout;
  OutPort<TimedDouble> m_doubleoutOut;

  TimedShortSeq m_sarrout;
  OutPort<TimedShortSeq> m_sarroutOut;
  TimedLongSeq m_larrout;
  OutPort<TimedLongSeq> m_larroutOut;
  TimedFloatSeq m_farrout;
  OutPort<TimedFloatSeq> m_farroutOut;
  TimedDoubleSeq m_darrout;
  OutPort<TimedDoubleSeq> m_darroutOut;

  int num, type, plus;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_PrvPortPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  summaterSVC_impl m_summater;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;

};


extern "C"
{
  void SummaterInit(RTC::Manager* manager);
};

#endif // SUMMATER_H
